# !/usr/bin/env python3
import requests
import json
import datetime
from time import strftime

subscription_id = "
client_id = "
client_secret = "
tenant_id = 


def get_token():
    """ Gets the authorization token for accessing azure.

    Returns:
        str: Azure access token
    """
    microsoft_token_url = 'https://login.microsoftonline.com/{}/oauth2/token'.format(tenant_id)
    # make the request for authorization token
    response = requests.post(
        microsoft_token_url,
        headers={
            "Content-Type": "application/x-www-form-urlencoded"
        },
        data={
            "client_id": client_id,
            "client_secret": client_secret,
            "grant_type": "client_credentials",
            "resource": "https://management.azure.com"
        }
    )
    if response.status_code == 200:
        return(response.json()['access_token'])
    else:
        raise Exception("Bad credentials")

def check_backup_attempts(token):
        """ Checks the status of the Siterra Azure backups.
            Its checking to see if at least 2 backups ran in the last 12 hours (GMT time).

        Args:
            token(str): Azure authorization token

        Returns:
            int: status code
                0- all good
                1- failure
        """

        current = datetime.datetime.utcnow()
        half_day = datetime.timedelta(hours=12)
        poll_dt = current - half_day
        poll_date = str(poll_dt.date())
        poll_time = str(poll_dt.strftime("%H:%M:%S"))

        monitor_status = ''
        url = "https://management.azure.com/subscriptions/" + subscription_id + "/resourceGroups/" \
              "acc_apim_prod_backup/providers/Microsoft.Logic/workflows/" \
              "acc-apim-prod-synchronization/runs?api-version=2016-06-01&$filter=startTime ge " + \
              poll_date + "T" + poll_time + "Z"
        payload = {}
        headers = {
            'Authorization': 'Bearer {}'.format(access_token)
        }
        response = requests.request("GET", url, headers=headers, data=payload)

        monitor_json = json.loads(response.text)
        count = 0
        for i in monitor_json["value"]:
            if 'properties' in i.keys():
                count += 1
        if count > 1:
            monitor_status = 1
        else:
            monitor_status = 0
        return monitor_status

if __name__ == "__main__":
    access_token = get_token()
    print(check_backup_attempts(access_token))
